@@ -1,3 +1,6 @@
+#if !defined(HFUNC_H)
+#define HFUNC_H
+
 // File hfunc.c
 void Hfunc1(int* family,int* n,double* u,double* v,double* theta,double* nu,double* out);
 void Hfunc2(int* family,int* n,double* v,double* u,double* theta,double* nu,double* out);
@@ -10,3 +13,8 @@ void pcondbb1(double* u, double* v, int* n, double* param, double* out);
 void pcondbb6(double* u, double* v, int* n, double* param, double* out);
 void pcondbb7(double* u, double* v, int* n, double* param, double* out);
 void pcondbb8(double* u, double* v, int* n, double* param, double* out);
+void qcondgum(double* p, double* z1, double* de, double* out);
+void qcondjoe(double* q, double* u, double* de, double* out);
+//void qcondbb1(double* q, double* u, double* de, double* th, double* out);
+
+#endif
